
 /* DMA Driver 
  * By Tyler Olivieri
  * 
  * This driver is for DARAM to DARAM transfers using the DMA
  * Sync events automatically turned off 
  * 
  * */

   /* Dma_init function 
    * used to set internal data of the struct
    * the base address is the CPU word address for DMACHnSSAL (in hex)
    */
    
   #include "dma_driver.h"
   #include "Application_1_Modified_Registers.h"
   
    
   Int16 Dma_init(Dma_Obj * hDma,  Uint16 base_Address, dmaTransferMode mode){
   	 
   		//Check to see if struct pointer is NULL
   		if(hDma == NULL){
   	 		return (0);
   	 	}
   	 	//set the internal data of the struct 
	 	hDma-> baseAddress =  base_Address;
	 	hDma->mode = mode;
	 	
	 	return (1);
   	 
   }
   
   
   /*
    *  Dma_Config function 
    *  Sets the sourceAddress, destination Address and transfer length
    *  Pass pointer to the source buffer and dest buffer 
    *  Interrupts are enabled
    *  transfer length is in bytes -- specify to be number of samples to transfer
    */ 
   Int16 Dma_Config(Dma_Obj * hDma, Int16 * sourceAddress, Int16 * destAddress, Uint16 transferLength){ 
   		
   		Uint16 base_address = hDma->baseAddress;
   		dmaTransferMode mode = hDma->mode;
   		Uint32 dma_address;
   		
   		//Check to see if struct pointer is NULL
   		if(hDma == NULL){
   			return (0);
   		} 	
   
   		if(mode == DARAM2DARAM){
	   		//Convert the source Address from CPU word address to DMA byte address
	   		//This is done by muliplying by 2; This occurs because CPU is two byte addressable 
	   		//Then an offset is added; 0x10000 in this case for DARAM transfers
			dma_address = (Uint32)sourceAddress;
			dma_address = (dma_address<<1) + DARAMoffset;
			
			//store the source address into the dma struct --maybe don't need to do this
			
			//Seperate into upper and lower 2bytes and configure DMA from the base address 
			//write into the registers 
			ACCESS_REG(sourceStartL) = (Uint16)dma_address;//(Uint16)dma_address;
			ACCESS_REG(sourceStartU) = 0xFFFF & (dma_address >> 16);
			
		    //do the same for destination Address
		    dma_address = (Uint32)destAddress;
			dma_address = (dma_address<<1) + DARAMoffset;
			ACCESS_REG(destStartL) = (Uint16)dma_address;
			ACCESS_REG(destStartU) = 0xFFFF & (dma_address >> 16);
			
			//store the transfer length and set the transfer control registers
			
			//Set TCR1 to transfer length
			ACCESS_REG(transferControl1) =  transferLength*2;
			
			//Set TCR2 to no sync, increment source and destination, auto reload, no ping pong, 4 byte transfers, CPU interrupt enabled
		    ACCESS_REG(transferControl2) = 0x2000;
   		}
   		
   		// source is constant, destination address increments by four bytes after each transfer.
   		if(mode == destDARAM){
   		
   			ACCESS_REG(sourceStartL) = (Uint16) sourceAddress;
   			ACCESS_REG(sourceStartU) = (Uint16) 0x0000;
   			dma_address = (Uint32)destAddress;
			dma_address = (dma_address<<1) + DARAMoffset;
			ACCESS_REG(destStartL) = (Uint16)dma_address;
			ACCESS_REG(destStartU) = 0xFFFF & (dma_address >> 16);
			ACCESS_REG(transferControl1) =  transferLength*2;
			ACCESS_REG(transferControl2) = 0x3084;
   		}
   		
   		// destination is constant, source address increments by four bytes after each transfer.
   		if(mode == sourceDARAM){
   		
   			dma_address = (Uint32)sourceAddress;
			dma_address = (dma_address<<1) + DARAMoffset;
			ACCESS_REG(sourceStartL) = (Uint16)dma_address;
			ACCESS_REG(sourceStartU) = 0xFFFF & (dma_address >> 16);
			ACCESS_REG(destStartL) = (Uint16) destAddress;
   			ACCESS_REG(destStartU) = (Uint16) 0x0000;
   			ACCESS_REG(transferControl1) =  transferLength*2;
			ACCESS_REG(transferControl2) = 0x3204;
   				
   		}
	    return(1);
   			
   }
  
  	/* Dma_start will initatiate the data transfer
  	 * Interrupt will be thrown when transfer is complete
  	 * DMA_ISR must be registered in the interrupt vector table
  	 * See CSL_IRQ
  	 */
  
	Int16 Dma_start(Dma_Obj * hDma){
	    
		ACCESS_REG(transferControl2) = ACCESS_REG(transferControl2) | 0x8000;
		return(1);
	}
  
 	/* Dma_interrupts clears all interrupt flags 
 	 * Initalizes interrupts for all dmas and channels...
 	 */ 
 	 
  	Int16 Dma_interrupts(){
	
		//Clears all interrupts; writing a 1 clears the bit
		DMA_IFR = 0xFFFF;	  	
		
		//Clear DMA interrupt flags in the CPU interrupt flag registers 
		IFR0 = 0x0100;   
  	
  		//Enable interrupts for all dmas and all channels
  		DMA_IER = 0xFFFF;
  	
  		return(1);
  	}
  	
  	Int16 Dma_enablePingPong (Dma_Obj * hDma){
  		
  			ACCESS_REG(transferControl2) = ACCESS_REG(transferControl2) | 0x0001;
  			return(1);
  	}	
  	
  	Int16 Dma_enable_I2S(){
  		
  		Uint16 register_value;
	  	register_value 	= DMA1_CESR1;
	    DMA1_CESR1 		= register_value | 0x0202; /* Set CH1, CH0 sync event to I2S2 receive */
	    register_value 	= DMA1_CESR2;
	    DMA1_CESR2 		= register_value | 0x0101; /* Set CH3, CH2 sync event to I2S2 transmit*/
	    return(1);
  	}